@@ -15,6 +15,14 @@ use Tree::DAG_Node;
 
 use Types::Standard qw/Any Bool Int Object Str/;
 
+has clean_nodes =>
+(
+	default  => sub{return 0},
+	is       => 'rw',
+	isa      => Bool,
+	required => 0,
+);
+
 has attributes =>
 (
 	default  => sub{return 0},
@@ -87,7 +95,7 @@ has verbose =>
 	required => 0,
 );
 
-our $VERSION = '1.01';
+our $VERSION = '1.02';
 
 # ------------------------------------------------
 
@@ -125,6 +133,59 @@ sub _add_daughter
 
 } # End of _add_daughter.
 
+# --------------------------------------------------
+
+sub clean_tree
+{
+	my($self) = @_;
+
+	my($attributes);
+	my($key);
+	my($name);
+	my($value);
+
+	$self -> root -> walk_down
+	({
+		callback => sub
+		{
+			my($node, $opt) = @_;
+			$name           = $node -> name;
+			$attributes     = $node -> attributes;
+
+			# For for name is undef.
+
+			$node -> name('') if (! defined $name);
+
+			# Fix for attribute name is undef.
+
+			if (exists $$attributes{undef})
+			{
+				if (! exists $$attributes{''})
+				{
+					$$attributes{''} = $$attributes{undef};
+				}
+
+				delete $$attributes{undef};
+			}
+
+			# Fix for attribute value is undef.
+
+			for $key (keys %$attributes)
+			{
+				$value             = $$attributes{$key};
+				$value             = '' if (! defined $value);
+				$$attributes{$key} = $value;
+			}
+
+			$node -> attributes($attributes);
+
+			return 1; # Keep walking.
+		},
+		_depth => 0,
+	});
+
+} # End of clean_tree.
+
 # ------------------------------------------------
 
 sub _process_arrayref
@@ -380,6 +441,18 @@ sub process_tree
 # ------------------------------------------------
 
 sub render
+{
+	my($self, $s) = @_;
+
+	$self -> run($s);
+
+	return $self -> root -> tree2string({no_attributes => 1 - $self -> attributes});
+
+} # End of render.
+
+# ------------------------------------------------
+
+sub run
 {
 	my($self, $s) = @_;
 	$s = defined($s) ? $s : 'undef';
@@ -421,6 +494,7 @@ sub render
 	}
 
 	$self -> process_tree;
+	$self -> clean_tree if ($self -> clean_nodes);
 
 	# Clean up in case user reuses this object.
 
@@ -429,9 +503,7 @@ sub render
 	$self -> node_stack -> pop if ($bless_type);
 	$self -> uid(0);
 
-	return $self -> root -> tree2string({no_attributes => 1 - $self -> attributes});
-
-} # End of render.
+} # End of run.
 
 # ------------------------------------------------
 
@@ -496,6 +568,8 @@ This is scripts/synopsis.pl:
 
 	print join("\n", @$result), "\n";
 
+See the L</FAQ> for a discussion as to whether you should call L</render($s)> or L</run($s)>.
+
 This is the output of scripts/synopsis.pl:
 
 	Synopsis
@@ -630,6 +704,15 @@ output.
 
 Default: 0.
 
+=item o clean_nodes => $Boolean
+
+Clean up nodes before printing, by changing the node's name to '' (the empty string) if it's undef,
+and by changing to '' any node attribute value which is undef.
+
+Set to 1 to activate option.
+
+Default: 0.
+
 =item o max_key_length => $int
 
 Use this to limit the lengths of hash keys.
@@ -662,6 +745,38 @@ Note: The value passed to L<Tree::DAG_Node>'s C<tree2string()> method is (1 - $B
 
 C<attributes> is a parameter to L</new()>.
 
+=head2 clean_nodes([$Boolean])
+
+Gets or sets the value of the C<clean_nodes> option.
+
+This stops undef warnings when printing (i.e. when calling the tree's tree2string() method).
+
+Values:
+
+=over 4
+
+=item o 0
+
+Do not clean up nodes.
+
+=item o 1
+
+Clean up nodes by doing both these:
+
+=over 4
+
+=item o The node's name
+
+Change the node's name to '' (the empty string) if it's undef.
+
+=item o The node's attribute values
+
+Change to '' any node attribute value which is undef.
+
+=back
+
+=back
+
 =head2 max_key_length([$int])
 
 Here, the [] indicate an optional parameter.
@@ -690,11 +805,7 @@ various bits of data attached to each node in the tree.
 If sub-classing this module, e.g. to change the precise text displayed, I recommend concentrating
 your efforts on this method.
 
-Alternately, see the answer to the first question in the L</FAQ>.
-
-=head2 root()
-
-Returns the root node in the tree, which is an object of type L<Tree::DAG_Node>.
+Also, see the answer to the first question in the L</FAQ>.
 
 =head2 render($s)
 
@@ -704,6 +815,24 @@ Returns an arrayref after calling the C<tree2string()> method for L<Tree::DAG_No
 
 See L</Synopsis> for a typical usage.
 
+See also L</run($s)>, which returns the root of the tree.
+
+The choice of calling C<render()> or C<run()> is discussed in the L</FAQ>.
+
+=head2 root()
+
+Returns the root node in the tree, which is an object of type L<Tree::DAG_Node>.
+
+=head2 run($s)
+
+Renders $s into an object of type L<Tree::DAG_Node>.
+
+Returns the root of the tree.
+
+See also L</render($s)>, which returns an arrayref of the tree's data.
+
+The choice of calling C<render()> or C<run()> is discussed in the L</FAQ>.
+
 =head2 title([$s])
 
 Here, the [] indicate an optional parameter.
@@ -725,12 +854,32 @@ C<verbose> is a parameter to L</new()>.
 
 =head2 Can I process the tree myself?
 
-Sure. Just call L</root()> - after L</render()> - to get the root of the tree, and process it any
-way you wish.
+Sure. Just call L</render($s)> and then L</root($s)> to get the root of the tree, and process it
+any way you wish.
+
+Otherwise, call L</root($s)>, which returns the root directly.
+
+See the next question for help in choosing between C<render()> and C<root()>.
 
 See L</process_tree()> for sample code. More information is in the docs for L<Tree::DAG_Node>
 especially under the discussion of C<walk_down()>.
 
+=head2 How do I choose between calling C<render()> and C<root()>?
+
+L</render($s)> automatically calls L<Tree::DAG_Node>'s C<tree2string()> method, which you may not
+want.
+
+For instance, the tree might have undef as the name of node, or the value of an attribute, so that
+calling C<tree2string()> triggers undefined variable warnings when converting the node's attributes
+to a string.
+
+With L</run($s)>, since C<tree2string()> is not called, you have time to process the tree yourself,
+cleaning up the nodes, before converting it to an arrayref of strings by calling C<tree2string()>
+directly. Indeed, with C<run()> you may never call C<tree2string()> at all.
+
+Users of L<Marpa::R2> would normally call L</run($s)>. See L<MarpaX::Languages::Lua::Parser> for
+sample code, where node names of undef are the reason for this choice.
+
 =head2 What are the attributes of the tree nodes?
 
 Firslty, each node has a name, which you can set or get with the C<name([$new_name])> method. Here,
@@ -838,8 +987,8 @@ Why is there a '1 = []' in there?
 
 Firstly, note that C<hash> keys are returned in sorted order.
 
-And, for C<hash> keys, that integer counts them, so C<A> would have gotten a 0, and C<C> would get a
-2, if they pointed to arrayrefs.
+And, for C<hash> keys, that integer counts them, so C<A> would have gotten a 0, and C<C> would get
+a 2, if they pointed to arrayrefs.
 
 =head2 Why did you use Text::Truncate?
 
